/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Forecastfox.
 *
 * The Initial Developer of the Original Code is
 * Jon Stritar <jstritar@MIT.EDU>.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Jon Stritar <jstritar@MIT.EDU>
 * Richard Klein <richwklein@mchsi.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

const CLASS_ID = Components.ID("{6E33307F-98BF-42da-98CA-DCDED4282E84}");
const CLASS_NAME = "Forecastfox Manager Component";
const CONTRACT_ID = "@ensolis.com/forecastfox/manager;1";
const ffIError = Components.interfaces.ffIError;
/******************************************************************************
 * ffManager component
 ******************************************************************************/
function ffManager()
{
  this.wrappedJSObject = this;
}

ffManager.prototype = {
  _partner: "forecastfox",
  _url: "http://forecastfox.accuweather.com/adcbin/forecastfox/weather_data.asp?",
  _ping: null,
  _parser: null,
  _profiles: null,
  _firstTime: true,
  _groups: null,
  _branch: null,
  _obs: null,
  _timer: null,
  _bundle: null,
  _requests: null,
  _persist: null,
   
  //readonly attributes fo UI
  get partner() { return this._partner; },
  get url() { return this._url; },
  get parser() { return this._parser; }, 
  get firstTime() { return this._firstTime; },
  get profiles() { return this._profiles; },

  load: function(aForced)
  {     
    //initialize
    if (this._firstTime) {
      //set not first time
      this._firstTime = false;     
      this.start();
    }
          
    //reset last prefs if forced
    for (i in this._groups) {
      if (aForced && this._branch.prefHasUserValue(this._groups[i].last))
        this._branch.clearUserPref(this._groups[i].last);
    };
            
    //get data
    this._getData();
  },

  getError: function(aGroup)
  {
    var error;
    if (!this._groups[aGroup]) {
      error = Components.classes["@ensolis.com/forecastfox/error;1"].createInstance(Components.interfaces.ffIError);
      error.init(ffIError.SEVERITY_INFO, "@ensolis.com/forecastfox/manager;1", "", "");
    } else
      error = this._groups[aGroup].error;
      
    return error;
  },
  
  start: function()
  {
    //set components
    var pbs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService)
    var sbs = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
    this._ping = Components.classes["@ensolis.com/forecastfox/ping;1"].createInstance(Components.interfaces.ffIPing);
    this._parser = Components.classes["@ensolis.com/forecastfox/parser;1"].createInstance(Components.interfaces.ffIParser);
    this._profiles = Components.classes["@ensolis.com/forecastfox/profiles;1"].createInstance(Components.interfaces.ffIProfiles);
    this._obs = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
    this._timer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
    this._branch = pbs.getBranch("forecastfox.");
    this._bundle = sbs.createBundle("chrome://forecastfox/locale/forecastfox.properties");
        
    //startup components
    this._ping.start();
    this._parser.start();
    
    try {
      var restart = this._profiles.start();
      if (restart.severity != ffIError.SEVERITY_INFO) {
        var ps = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);    
        ps.alert(this._getWindow(), restart.description, restart.tooltip);   
      };
    } catch (e) { }
  
    //setup current group
    this._groups = new Object();
    this._groups["current"] = new Object();
    this._groups["current"].targets = ["swa", "radar", "current"];
    this._groups["current"].frequency = 30;
    this._groups["current"].last = "cc.last";
    this._groups["current"].cache = "cc.cache";
    this._groups["current"].from = "server";
    this._groups["current"].error = Components.classes["@ensolis.com/forecastfox/error;1"].createInstance(Components.interfaces.ffIError);
    this._groups["current"].loading = false;
  
    //setup forecast group
    this._groups["forecast"] = new Object();
    this._groups["forecast"].targets = ["days", "nights"];
    this._groups["forecast"].frequency = 120;
    this._groups["forecast"].last = "dayf.last";
    this._groups["forecast"].cache = "dayf.cache";
    this._groups["forecast"].from = "server";
    this._groups["forecast"].error = Components.classes["@ensolis.com/forecastfox/error;1"].createInstance(Components.interfaces.ffIError);
    this._groups["forecast"].loading = false;
  	  
    //add shutdown observer
    this._obs.addObserver(this, "xpcom-shutdown", false);
    this._obs.addObserver(this, "forecastfox:profile", false);

    //add prefernce observer
    var pbi = this._branch.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
    pbi.addObserver("", this, false);
  },
  
  stop: function()
  {
  
    //cancel request
    this._cancelData();
     
    //stop timers
    this._timer.cancel();
    for (var j=0; j<this._requests.length; j++)
      this._requests[j].timeout.cancel();
                  
    //clean up held observers etc to avoid leaks.
    try {
      this._obs.removeObserver(this, "xpcom-shutdown");
      this._obs.removeObserver(this, "forecastfox:profile");
      var pbi = this._branch.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
      pbi.removeObserver("", this);
    } catch(e) {};
              
    //stop components
    this._ping.stop();
    this._parser.stop();
    this._profiles.stop();
        
    //destroy variables
    this._ping = null;
    this._parser = null;
    this._profiles = null;
    this._partner = null;
    this._url = null;
    this._firstTime = null;
    this._groups = null;
    this._branch = null;
    this._obs = null;
    this._timer = null;
    this._bundle = null;
    this._requests = null;
    this._persist = null;  
  },
    
  _getData: function()
  {
    //do nothing if location not set
    if (!this._checkLocID())
      return;

    for (i in this._groups) {
      //notify observers of update
      this._obs.notifyObservers(this, "forecastfox:manager", "start," + i);
      
      //check if we need to come from server
      this._checkServer(i);

      //clear last error
      this._groups[i].error.init(ffIError.SEVERITY_INFO, "@ensolis.com/forecastfox/error;1", "", "");
    
      //mark for loading
      this._groups[i].loading = true;
    };
    
    //build out our data requests
    this._requestData();
  },
  
  _requestData: function()
  {
    var cache = new Object();
    var server = false;
    var j, request, query;
    var component = this;

    //cancel any requests
    this._cancelData();
        
    //build new requests
    this._requests = new Array();    
    for (i in this._groups) {
      if (this._groups[i].from == "server") {
        if (!server) {
          query = this._getServerURL();
          request = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Components.interfaces.nsIXMLHttpRequest);
          request.timeout = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
          request.overrideMimeType("text/xml");
          request.from = "server";
          request.error = false;
          request.onerror = function() { component._receiveData(this); };            
          request.onload = function() { component._receiveData(this); };            
          request.open("GET", query, true);            
          this._requests.push(request);
          server = true;                 
        };
      } else {
        if (!cache[this._groups[i].from]) {
          query = this._getFileURL(this._groups[i].from);
          request = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Components.interfaces.nsIXMLHttpRequest);
          request.timeout = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
          request.overrideMimeType("text/xml");
          request.from = this._groups[i].from;
          request.error = false;          
          request.onerror = function() { component._receiveData(this); };            
          request.onload = function() { component._receiveData(this); };            
          request.open("GET", query, true);            
          this._requests.push(request);
          cache[this._groups[i].from] = true;              
        };
      };
    };
    
    //send requests
    for (j=0; j<this._requests.length; j++) {
      this._requests[j].timeout.initWithCallback(this, 20 * 1000, Components.interfaces.nsITimer.TYPE_ONE_SHOT);
      this._requests[j].send(null);
    }
  },
  
  _receiveData: function(aXMLReq)
  {
    var targets, j;
    var parseError = false;
    var hasError = false;
    
    //cancel timeout
    aXMLReq.timeout.cancel();
    
    for (i in this._groups) {
    
      //skip if not the same from
      if (this._groups[i].from != aXMLReq.from)
        continue;
      
      //mark loading complete
      this._groups[i].loading = false;
      
      //check if we actual received anything
      if (aXMLReq.responseText == "") {
        this._setError(i, ffIError.SEVERITY_ERROR, "ff.error.doc", null, null, null);
        hasError = true;          
        continue; 
      };
                 
      //check status if from server
      if (aXMLReq.from == "server") {
        if (aXMLReq.status == 407) {
          //XXX do authentication
          this._setError(i, ffIError.SEVERITY_ERROR, "ff.error.auth", null, null, null);
          hasError = true;
          continue;
        };  
        if (aXMLReq.status > 200) {
          this._setError(i, ffIError.SEVERITY_ERROR, "ff.error.net", null, null, null);
          hasError = true;
          continue;
        };
      };
      
      //parse document
      targets = this._groups[i].targets;
      for (j=0; j<targets.length; j++) {
        error = this._parser.open(aXMLReq.responseXML, targets[j], (j==0));
        
        //check for error
        if (error.severity != ffIError.SEVERITY_INFO) {
          this._setError(i, ffIError.SEVERITY_ERROR, null, null, null, error);
          hasError = true;
          parseError = true;      
          break;
        };
      };
      
      //go to next group
      if (parseError)
        continue;
      
      //do notifications when from server
      if (aXMLReq.from == "server" && i == "current") {
          //save radar image
          this._radar();
            
          //do sliders
          this._slider();
      };
      
      //notify update complete
      if (i != "current" || (i == "current" && aXMLReq.from != "server"))
        this._obs.notifyObservers(this, "forecastfox:manager", "end," + i);
    };
    
    //had errors
    if (hasError) {
      aXMLReq.error = true;
       
      //force from server next time
      if (aXMLReq.from != "server") {
        for (i in this._groups) {
          if (this._groups[i].from == aXMLReq.from && 
              this._branch.prefHasUserValue(this._groups[i].last))
            this._branch.clearUserPref(this._groups[i].last);            
        };
      };      
    };
        
    //save data 
    if (aXMLReq.from == "server")
      this._saveData(aXMLReq.responseText);    

    //schedule
    switch (this._checkSchedule()) {
      case "none":
        break;
      case "error":
        this._schedule(2*60*1000);
        break;
      case "normal":
      default:
        this._schedule();
        break;
    }
  },
  
  _cancelData: function()
  {  
    //cancel pending persists
    if (this._persist) {
      if (this._persist.currentState == 2)
        this._persist.cancelSave(); 
    };
      
    //skip if no requests
    if (!this._requests)
      return;

    //loop through groups
    for (i in this._groups) {
      //match request with group
      for (var j=0; j<this._requests.length; j++) {
        if (this._groups[i].from != this._requests[j].from)   
          continue;
        
        //cancel any pending requests
        if (this._requests[j].readyState < 4) {
          this._requests[j].abort();
          this._requests[j].timeout.cancel();
          this._obs.notifyObservers(this, "forecastfox:manager", "end," + i);
          break;  
        };
      };    
    }
  },
  
  _radar: function()
  {
    var ios = Components.classes["@mozilla.org/network/io-service;1"].createInstance(Components.interfaces.nsIIOService);
    var name = "radarcache-" + this._profiles.current + ".gif";
    var source = this._parser.getValue("radar", "image", null);
 
 	if (source != "") {
      //convert url to uri and get file
      var uri = ios.newURI(source, null, null);
      var file = this._getFile(name, "create");

      var flags = 0x02 | 0x32;
      this._persist = Components.classes["@mozilla.org/embedding/browser/nsWebBrowserPersist;1"].createInstance(Components.interfaces.nsIWebBrowserPersist); 				      
      this._persist.persistFlags |= flags;
      this._persist.progressListener = this;
      this._persist.saveURI(uri, null, null, null, null, file);
      
      //set cache
      this._branch.setCharPref("radar.cache", name);
    } else {
      this._branch.setCharPref("radar.cache", "");
      this._obs.notifyObservers(this, "forecastfox:manager", "end,current");
    }
    
  },
  
  _slider: function()
  {
    var type = null;
    var freq = 1;
    var count = 1;
    var enabled = false;
    var active = false;
    var icon = "";
    var title = "";
    var text = "";
    var url = "";

    //always increment count
    count = this._branch.getIntPref("cc.slider.count");
    this._branch.setIntPref("cc.slider.count", count + 1);
              
    //determine if we are showing swa
    enabled = this._branch.getBoolPref("swa.slider.enabled");
    if (this._parser.getValue("swa", "active", null) == "true")
      active = true;
    else
      active = false;
 
    if (enabled && active)
      type = "swa";
    
    //check if we are doing current
    if (!type) {
      enabled = this._branch.getBoolPref("cc.slider.enabled");
      freq = this._branch.getIntPref("cc.slider.freq");     
      if (enabled && count >= freq)
        type = "current";
    };
    
    //do neither
    if (!type)
      return;
         
    //set values based on type
    if (type == "current") {
      this._branch.setIntPref("cc.slider.count", 1);    
      url = this._parser.getValue("current", "url", null);
      switch (this._branch.getIntPref("cc.slider.display")) {
        case 0:
          icon = "chrome://forecastfox/skin/images/large/" + this._parser.getValue("current", "icon", null) + ".png";  
          break;
        case 1:
          title = this._parser.setLabel(this._branch.getComplexValue("cc.slider.title",Components.interfaces.nsIPrefLocalizedString).data, "current", null);
          text = this._parser.setLabel(this._branch.getComplexValue("cc.slider.label",Components.interfaces.nsIPrefLocalizedString).data, "current", null);
          break;
        case 2:
          icon = "chrome://forecastfox/skin/images/large/" + this._parser.getValue("current", "icon", null) + ".png";  
          title = this._parser.setLabel(this._branch.getComplexValue("cc.slider.title",Components.interfaces.nsIPrefLocalizedString).data, "current", null);
          text = this._parser.setLabel(this._branch.getComplexValue("cc.slider.label",Components.interfaces.nsIPrefLocalizedString).data, "current", null);
      };
    } else {   
      url = this._parser.getValue("swa", "url", null);
      title = this._parser.setLabel(this._branch.getComplexValue("swa.slider.title",Components.interfaces.nsIPrefLocalizedString).data, "swa", null);
      text = this._parser.setLabel(this._branch.getComplexValue("swa.slider.label",Components.interfaces.nsIPrefLocalizedString).data, "swa", null);      
          icon = "chrome://forecastfox/skin/images/large/" + this._parser.getValue("swa", "icon", null) + ".png";  
    };

    //do alert  
    try {
      var alerts = Components.classes["@mozilla.org/alerts-service;1"].getService(Components.interfaces.nsIAlertsService);
      alerts.showAlertNotification(icon, title, text, true, url, this);
    } catch(e) {};         
  },
    
  _saveData: function(aText)
  {
    var freq = 0;
    var profile = this._profiles.current;
    var current = (new Date()).valueOf();
    var name;
    
    //figure out file name
    for (i in this._groups) {
      if (this._groups[i].from != "server")
        continue;
        
      if (this._groups[i].frequency >= freq) {
        freq = this._groups[i].frequency;
        name = i + "cache-" + profile + ".xml"; 
      };
    };
    
    //get file interface    
    var file = this._getFile(name, "create");

    //flush data to disk       
    var flags = 0x02 | 0x08 | 0x20;
    var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);    
    foStream.init(file, flags, 0664, 0);
    foStream.write(aText, aText.length);
    foStream.close();
    
    //update the preferences    
    for (i in this._groups) {
      if (this._groups[i].from != "server")
        continue;
      this._branch.setCharPref(this._groups[i].cache, name);
      this._branch.setCharPref(this._groups[i].last, current);
    }    
  },
    
  _setError: function(aGroup, aSeverity, aName, aDescription, aTooltip, aError)
  {
    if (!aError) {
      var description = (aName) ? this._bundle.GetStringFromName(aName + ".label") : aDescription;
      var tooltip = (aName) ? this._bundle.GetStringFromName(aName + ".tooltip") : aTooltip;
      this._groups[aGroup].error.init(aSeverity, "@ensolis.com/forecastfox/manager;1", description, tooltip); 
    } else
      this._groups[aGroup].error = aError; 
      
    this._obs.notifyObservers(this, "forecastfox:manager", "error," + aGroup);       
  },
  
  _getWindow: function()
  {
    //XXX need app specific code for doing this   
    var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
    var top = wm.getMostRecentWindow("navigator:browser");
    if (!top)
      top = window.openDialog("chrome://browser/content/browser.xul", "_blank", "chrome,all,dialog=no", "about:blank", null, null);
    return top;  
  },
  
  _checkSchedule: function()
  {    
    //first check all requests are complete
    for (i=0; i<this._requests.length; i++) {
      if (this._requests[i].readyState < 4)
        return "none";
    };
    
    //now check if we had any errors
    for (i=0; i<this._requests.length; i++) {
      if (this._requests[i].error)
        return  "error";
    };

    return "normal";
  },
  
  _checkLocID: function()
  {
    var isSet = this._branch.prefHasUserValue("general.locid");
    var locid = this._branch.getCharPref("general.locid");
      
    //if the locid hasn't been set yet, prompt for
    if (!isSet || locid == "00000" || locid == "") {
      var wm = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
      var options = wm.getMostRecentWindow("forecastfox:options");
      if (options)
        options.focus();
      else    
        this._getWindow().openDialog("chrome://forecastfox/content/options.xul", "options", "chrome,modal");    
      return false; 
    };
    
    return true;
  },

  _checkServer: function(aName)
  {
    //current date/time
    var current = (new Date()).valueOf();

    //then get the time for the next refresh.
    var last = Number(this._branch.getCharPref(this._groups[aName].last));
    var next = last + (this._groups[aName].frequency * 60 * 1000);

	//refresh if it has been the allowed time, or if the last refresh occured in the future
    if ((last > current) || ((next - current) <= 100))
      this._groups[aName].from = "server";
    else if (this._checkCache(aName))
      this._groups[aName].from = this._branch.getCharPref(this._groups[aName].cache);
    else
      this._groups[aName].from = "server";
  },
  
  _checkCache: function(aName)
  {
    var pref = this._branch.getCharPref(this._groups[aName].cache);
    
    if (!pref)
      return false;
      
    if (!this._getFileURL(pref))
      return false;
    
    return true;
  },
      
  _getFileURL: function(aName)
  {
    var file = this._getFile(aName, "exists");
    if (!file)
      return null;
      
    //convert to a URI and return spec
    var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
    return ios.newFileURI(file).spec;  
  },
  
  _getServerURL: function()
  {
    var query = this._url + "location=" + this._branch.getCharPref("general.locid");

    for (i in this._groups) {
      if (this._groups[i].from == "server")
        query += "&" + i + "=1";
      else
        query += "&" + i + "=0";
    };
      
    //set units
    var units = this._branch.getIntPref("general.units");   
    if (units == 1)
      query += "&metric=0";
    else if (units == 2)
      query += "&metric=3";
    else
      query += "&metric=1";
    
    //add partner  
    query += "&partner=" + this._partner;
    
    //add version
    query += "&version=0.7.9";
    
    //return string
    return query
  },
  
  _schedule: function(aForce)
  {
    var actual = null;
    
    if (aForce)
      actual = aForce;
    else {
      var current = (new Date()).valueOf();
      var next = new Array();
      var span = this._branch.getIntPref("schedule.span") * 60 * 1000;
      var bump = this._branch.getIntPref("schedule.bump") /100;
      var value;
          
      //get time till next refresh for each category.
      for (i in this._groups) {
        if (this._groups[i].loading)
          return;
        value = Number(this._branch.getCharPref(this._groups[i].last));
        value += (this._groups[i].frequency * 60 * 1000);
        value -= current;
        next.push(value)
      }
      
      //sort by time
      next.sort(function (a, b) { return (a - b) });

      //if next is within span * bump go to next
      actual = next[0];
      for (var j=1; j<next.length; j++) {
        if (next[j] - actual < span) {
          actual = next[j];
          span *= bump;
        } else
          break;
      };
      
      //make sure we have a positive time
      if (actual < 0)
        actual = 100;

      actual += 100;
      
    };

    //cancel old timers
    this._timer.cancel();

    // Then schedule the timer
    this._timer.initWithCallback(this, actual, Components.interfaces.nsITimer.TYPE_ONE_SHOT);
  },
  
  _getFile: function(aName, aMode)
  {
    var dirs = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties);
    var folder = dirs.get("ProfD", Components.interfaces.nsIFile);
    
    //get forecastfox directory
    folder.append("forecastfox");
    if (!folder.exists())
      folder.create(folder.DIRECTORY_TYPE, 0755)
    
    //get file
    var file = folder.clone();      
    file.append(aName);
    
    //make sure its not a directory
    if (file.exists()) {   
      if (file.isDirectory()) 
        file.remove(true);
    };

    switch (aMode) {
      case "delete":
        if (file.exists())
          file.remove(false);
        file = null;
        break;
      case "exists":
        if (!file.exists())
          file = null;
        break;
      case "create":
      default:
        if (file.exists())
          file.remove(false);
        file.create(file.NORMAL_FILE_TYPE, 0664);             
        break;
    };  
   
    //make sure file is writable
    if (file) {
      if (!file.isWritable()) 
        file.permissions = 0644;    
    };
    
    //return file
    return file;
  },

  ///////////////////////////
  // nsITimerCallback
  notify: function(timer)
  {
    if (timer == this._timer)
      this._getData();
    else {
      for (var j=0; j<this._requests.length; j++) {
        if (this._requests[j].timeout != timer)
          continue;
        
        //abort the request    
        this._requests[j].abort();
        
        //send error and set to not loading
        for (i in this._groups) {
          if (this._groups[i].from == this._requests[j].from) {
            this._setError(i, ffIError.SEVERITY_ERROR, "ff.error.timeout", null, null, null);
            this._groups[i].loading = false;
          };
        };
      };
      
      //schedule update
      this._schedule(2*60*1000);
    }
  },
    
  ///////////////////////////
  // nsIWebProgressListener
  onLocationChange: function(webProgress, request, location) {},
  onProgressChange: function(webProgress, request, curSelfProgress, maxSelfProgress, curTotalProgress, maxTotalProgress ) {},
  onSecurityChange: function(webProgress, request, state) {},
  onStatusChange: function(webProgress, request, status, message) {},
  onStateChange: function(webProgress, request, stateFlags, status)
  {
    var stateDone = Components.interfaces.nsIWebProgressListener.STATE_STOP;
    if (stateDone == (stateFlags & stateDone))
      this._obs.notifyObservers(this, "forecastfox:manager", "end,current");
  },


  ///////////////////////////
  // nsIAlertListener
  onAlertFinished: function() {},
  onAlertClickCallback: function(aCookie)
  {
    //XXX need app specific
    var browser = this._getWindow().document.getElementById("content");
    browser.loadURI(aCookie);
  },

  /////////////////////////////
  // nsIObserver
  observe: function(aSubject, aTopic, aData)
  {
    var enabled, prefix;
    
    //do nothing until the first load
    if (this._firstTime)
      return;
      
    switch (aTopic) {
      case "forecastfox:profile":
        //profile loaded so send notification and schedule update
        this._obs.notifyObservers(this, "forecastfox:manager", "profile");
        this._schedule(200);
        break;        
      case "nsPref:changed":
        if (this._profiles.batch)
          return;              
        switch (aData) {      
          case "profile.current":
            this._profiles.load(this._branch.getCharPref(aData));
            break;
          case "profile.switch.enabled":
            enabled = this._branch.getBoolPref(aData);
            if (enabled)
              this._profiles.rotate(this._profiles.ROTATE_START);
            else
              this._profiles.rotate(this._profiles.ROTATE_STOP);
            break;
          case "profile.switch.delay":
            enabled = this._branch.getBoolPref("profile.switch.enabled");
            if (enabled)
              this._profiles.rotate(this._profiles.ROTATE_START);
            break;
          case "general.locid":
          case "general.units":
            for (i in this._groups) {
              if (this._branch.prefHasUserValue(this._groups[i].last))
                this._branch.clearUserPref(this._groups[i].last);
            };
            this._schedule(200);
            break;
          case "general.bar":
          case "general.position":   
            this._obs.notifyObservers(this, "forecastfox:manager", "move");
            break;
          case "radar.panel.enabled":
          case "swa.panel.enabled":
          case "cc.panel.enabled":
          case "dayt.panel.enabled":
          case "dayf.panel.enabled":             
            this._obs.notifyObservers(this, "forecastfox:manager", "enable");
            break;
          case "radar.panel.display":
          case "radar.panel.label":
          case "radar.tooltip.enabled":
          case "radar.tooltip.display":
          case "radar.tooltip.label":  
          case "swa.panel.display":
          case "swa.panel.label":
          case "swa.tooltip.enabled":
          case "swa.tooltip.display":
          case "swa.tooltip.label":                    
          case "cc.panel.display":
          case "cc.panel.label":
          case "cc.tooltip.enabled":
          case "cc.tooltip.display":
          case "cc.tooltip.label":
          case "dayt.panel.display":
          case "dayt.tooltip.enabled":
          case "dayt.tooltip.display":
          case "dayt.tooltip.label":
          case "dayt.panel.mode":
          case "dayt.panel.switch":
          case "dayt.panel.label":
          case "dayf.panel.display":
          case "dayf.panel.label":
          case "dayf.tooltip.enabled":
          case "dayf.tooltip.display":
          case "dayf.tooltip.label":
          case "dayf.panel.mode":
          case "dayf.panel.days":
            prefix = aData.substring(0, aData.indexOf("."));
            this._obs.notifyObservers(this, "forecastfox:manager", "data," + prefix);
            break;                       
        };            
        break;
      case "xpcom-shutdown":
		this.stop();
        break;
    }
  },

  QueryInterface: function (iid)
  {
    if (!iid.equals(Components.interfaces.ffIManager) &&
          !iid.equals(Components.interfaces.nsIAlertListener)
          && !iid.equals(Components.interfaces.nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
}

/******************************************************************************
 * XPCOM Functions for construction and registration
 ******************************************************************************/
var gModule = {
  _firstTime: true,
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    if (this._firstTime) {
      this._firstTime = false;
      throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
    };
    aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME, CONTRACT_ID, aFileSpec, aLocation, aType);
  },

  unregisterSelf: function(aCompMgr, aLocation, aType)
  {
    aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);        
  },
  
  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (aCID.equals(CLASS_ID))
      return gFactory;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  canUnload: function(aCompMgr) { return true; }
};

var gFactory = {
  createInstance: function (aOuter, aIID)
  {
    if (aOuter != null)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    return (new ffManager()).QueryInterface(aIID);
  }
};

function NSGetModule(aCompMgr, aFileSpec) { return gModule; }
